from __future__ import annotations

from typing import Any, Optional

from paddle.fluid.framework import Program as Program
from paddle.fluid.incubate.fleet.base.fleet_base import DistributedOptimizer, Fleet

from .optimizer_factory import *

class PSLib(Fleet):
    def __init__(self) -> None: ...
    def init(self, role_maker: Optional[Any] = ...) -> None: ...
    def set_pull_local_thread_num(self, thread_num: Any) -> None: ...
    all_ips_: Any = ...
    def init_worker(self) -> None: ...
    def init_server(self, model_dir: Optional[Any] = ..., **kwargs: Any) -> None: ...
    def run_server(self) -> None: ...
    def end_pass(self, scope: Any) -> None: ...
    def train_from_dataset(
        self,
        executor: Any,
        program: Optional[Any] = ...,
        dataset: Optional[Any] = ...,
        scope: Optional[Any] = ...,
        thread: int = ...,
        debug: bool = ...,
        fetch_list: Optional[Any] = ...,
        fetch_info: Optional[Any] = ...,
        print_period: int = ...,
        fetch_handler: Optional[Any] = ...,
    ) -> None: ...
    def start_heter_trainer(
        self,
        executor: Any,
        program: Optional[Any] = ...,
        scope: Optional[Any] = ...,
        debug: bool = ...,
        fetch_list: Optional[Any] = ...,
        fetch_info: Optional[Any] = ...,
        print_period: int = ...,
        fetch_handler: Optional[Any] = ...,
    ) -> None: ...
    def stop_worker(self) -> None: ...
    def distributed_optimizer(self, optimizer: Any, strategy: Any = ...): ...
    def save_inference_model(
        self,
        executor: Any,
        dirname: Any,
        feeded_var_names: Optional[Any] = ...,
        target_vars: Optional[Any] = ...,
        main_program: Optional[Any] = ...,
        export_for_deployment: bool = ...,
    ) -> None: ...
    def print_table_stat(self, table_id: Any) -> None: ...
    def set_file_num_one_shard(self, table_id: Any, file_num: Any) -> None: ...
    def save_persistables(
        self, executor: Any, dirname: Any, main_program: Optional[Any] = ..., **kwargs: Any
    ) -> None: ...
    def save_model_with_whitelist(
        self, executor: Any, dirname: Any, whitelist_path: Any, main_program: Optional[Any] = ..., **kwargs: Any
    ) -> None: ...
    def save_multi_table_one_path(self, table_ids: Any, model_dir: Any, **kwargs: Any) -> None: ...
    def save_cache_model(self, executor: Any, dirname: Any, main_program: Optional[Any] = ..., **kwargs: Any): ...
    def shrink_sparse_table(self) -> None: ...
    def shrink_dense_table(
        self, decay: Any, emb_dim: int = ..., scope: Optional[Any] = ..., table_id: Optional[Any] = ...
    ) -> None: ...
    def clear_one_table(self, table_id: Any) -> None: ...
    def clear_model(self) -> None: ...
    def load_pslib_whitelist(self, table_id: Any, model_path: Any, **kwargs: Any) -> None: ...
    def load_one_table(self, table_id: Any, model_path: Any, **kwargs: Any) -> None: ...
    def confirm(self) -> None: ...
    def revert(self) -> None: ...
    def load_model(self, model_dir: Optional[Any] = ..., **kwargs: Any) -> None: ...
    def save_model(self, model_dir: Optional[Any] = ..., **kwargs: Any) -> None: ...
    def save_one_table(self, table_id: Any, model_dir: Any, **kwargs: Any) -> None: ...
    def set_date(self, table_id: Any, date: Any) -> None: ...

fleet: Any

class fleet_embedding:
    origin_emb: Any = ...
    origin_emb_v2: Any = ...
    click_name: Any = ...
    scale_sparse_grad: Any = ...
    accessor: str = ...
    def __init__(self, click_name: Any, scale_sparse_grad: bool = ...) -> None: ...
    def __enter__(self) -> None: ...
    def __exit__(self, exc_type: Any, exc_val: Any, exc_tb: Any) -> None: ...

class DownpourOptimizer(DistributedOptimizer):
    def __init__(self, optimizer: Any, strategy: Optional[Any] = ...) -> None: ...
    def backward(
        self,
        loss: Any,
        startup_program: Optional[Any] = ...,
        parameter_list: Optional[Any] = ...,
        no_grad_set: Optional[Any] = ...,
        callbacks: Optional[Any] = ...,
    ) -> None: ...
    def apply_gradients(self, params_grads: Any) -> None: ...
    def get_dist_env(self): ...
    def minimize(
        self,
        losses: Any,
        scopes: Optional[Any] = ...,
        startup_programs: Optional[Any] = ...,
        parameter_list: Optional[Any] = ...,
        no_grad_set: Optional[Any] = ...,
        program_mode: str = ...,
    ): ...



from __future__ import annotations

from typing import Any, Optional

from ... import Tensor, nn

class ConvPoolLayer(nn.Layer):
    relu: Any = ...
    def __init__(
        self,
        input_channels: Any,
        output_channels: Any,
        filter_size: Any,
        stride: Any,
        padding: Any,
        stdv: Any,
        groups: int = ...,
        act: Optional[Any] = ...,
    ) -> None: ...
    def forward(self, inputs: Any) -> Tensor: ...

class AlexNet(nn.Layer):
    def __init__(self, num_classes: int = ...) -> None: ...
    def forward(self, inputs: Tensor) -> Tensor: ...
    __call__ = forward

def alexnet(pretrained: bool = ..., *, num_classes: int = ...) -> AlexNet: ...



from __future__ import annotations

from typing import Any

class ParallelMode:
    DATA_PARALLEL: int = ...
    TENSOR_PARALLEL: int = ...
    PIPELINE_PARALLEL: int = ...
    SHARDING_PARALLEL: int = ...

class CommunicateTopology:
    coordinate: Any = ...
    def __init__(self, hybrid_group_names: Any = ..., dims: Any = ...): ...
    def get_hybrid_group_names(self): ...
    def get_dim(self, axis_name: Any): ...
    def world_size(self): ...
    def get_rank(self, **args: Any): ...
    def get_coord(self, rank: Any): ...
    def get_axis_list(self, axis_name: Any, index: Any): ...
    def get_dim_size(self, axis_name: Any): ...
    def get_comm_list(self, axis_name: Any): ...
    def get_rank_from_stage(self, global_rank: Any, **kwargs: Any): ...

class HybridCommunicateGroup:
    nranks: Any = ...
    global_rank: Any = ...
    stage_id: Any = ...
    is_first_stage: Any = ...
    is_last_stage: Any = ...
    def __init__(self, topology: Any) -> None: ...
    def get_parallel_mode(self): ...
    def topology(self): ...
    def get_global_rank(self): ...
    def get_data_parallel_rank(self): ...
    def get_data_parallel_world_size(self): ...
    def get_data_parallel_group(self): ...
    def get_data_parallel_group_src_rank(self): ...
    def get_model_parallel_rank(self): ...
    def get_model_parallel_world_size(self): ...
    def get_model_parallel_group(self): ...
    def get_model_parallel_group_src_rank(self): ...
    def get_stage_id(self): ...
    def get_pipe_parallel_world_size(self): ...
    def get_pipe_parallel_group(self): ...
    def get_p2p_groups(self): ...
    def get_sharding_parallel_rank(self): ...
    def get_sharding_parallel_world_size(self): ...
    def get_sharding_parallel_group(self): ...
    def get_sharding_parallel_group_src_rank(self): ...
    def get_check_parallel_group(self): ...
    def get_rank_from_stage(self, stage_id: Any, **kwargs: Any): ...

class _CommunicateGroup:
    groups: Any = ...
    def __init__(self) -> None: ...
    def set_comm_group(
        self, group_name: Any, group_rank: Any, group_size: Any, ring_id: Any, group_ranks: Any
    ) -> None: ...
    def get_group(self, group_name: Any): ...
    def get_model_parallel_group(self): ...
    def get_model_parallel_world_size(self): ...
    def get_model_parallel_rank(self): ...



from __future__ import annotations

from typing import Any, Optional

from paddle.fluid.data_feeder import check_dtype as check_dtype
from paddle.fluid.data_feeder import (
    check_variable_and_dtype as check_variable_and_dtype,
)

from ..framework import core as core

class PrintOptions:
    precision: int = ...
    threshold: int = ...
    edgeitems: int = ...
    linewidth: int = ...
    sci_mode: bool = ...

DEFAULT_PRINT_OPTIONS: Any

def set_printoptions(
    precision: Optional[Any] = ...,
    threshold: Optional[Any] = ...,
    edgeitems: Optional[Any] = ...,
    sci_mode: Optional[Any] = ...,
    linewidth: Optional[Any] = ...,
) -> None: ...
def to_string(var: Any, prefix: str = ...): ...
def sparse_tensor_to_string(tensor: Any, prefix: str = ...): ...
def tensor_to_string(tensor: Any, prefix: str = ...): ...



from __future__ import annotations

from typing import Any, Optional

from .base_cost import OP_COST_FACTORY as OP_COST_FACTORY
from .base_cost import CompOpCost as CompOpCost
from .base_cost import Cost as Cost
from .base_cost import register_op_cost as register_op_cost

class MatmulV2OpCost(CompOpCost):
    OP_TYPE: str = ...
    def __init__(self, op: Optional[Any] = ..., op_desc: Optional[Any] = ..., cluster: Optional[Any] = ...) -> None: ...
    def calc_flops(self): ...
    def calc_time(self): ...



from __future__ import annotations

from typing import Any

from . import core as core
from . import unique_name as unique_name

MAX_INTEGER: Any

def is_list_tuple(index: Any, contain_type: Any): ...
def is_one_dim_list(index: Any, contain_type: Any): ...
def get_list_index_shape(var_dims: Any, index_dims: Any): ...

class SliceInfo:
    pre_shape: Any = ...
    indexes: Any = ...
    dtype: Any = ...
    def __init__(self) -> None: ...
    def update(self, index: Any) -> None: ...
    def shape_stride(self, shape: Any): ...
    def numel(self, shape: Any): ...
    def get_offset_stride(self, tensor_shape: Any): ...
    def get_item(self, tensor: Any): ...
    def set_item(self, tensor_origin: Any, value: Any): ...

def replace_ellipsis(var: Any, item: Any): ...
def replace_ndarray(item: Any): ...
def replace_none(item: Any): ...
def is_integer_or_scalar_tensor(ele: Any): ...
def is_bool_tensor(ele: Any): ...
def deal_attrs(attrs: Any, attr: Any, attr_name: Any, tensor_attr_name: Any, inputs: Any, infer_flags: Any) -> None: ...
def get_value_for_bool_tensor(var: Any, item: Any): ...
def set_value_for_bool_tensor(var: Any, item: Any, value: Any): ...



from __future__ import annotations

from typing import Any

class KVClient:
    endpoint: Any = ...
    def __init__(self, endpoint: str = ...) -> None: ...
    def put(self, key: Any, value: Any): ...
    def get(self, key: Any): ...
    def get_prefix(self, key: Any): ...
    def delete(self, key: Any): ...
    def wait_server_ready(self, timeout: int = ...): ...



from __future__ import annotations

from ..fluid.backward import append_backward as append_backward
from ..fluid.backward import gradients as gradients
from ..fluid.compiler import BuildStrategy as BuildStrategy
from ..fluid.compiler import CompiledProgram as CompiledProgram
from ..fluid.compiler import ExecutionStrategy as ExecutionStrategy
from ..fluid.compiler import IpuCompiledProgram as IpuCompiledProgram
from ..fluid.compiler import IpuStrategy as IpuStrategy
from ..fluid.executor import Executor as Executor
from ..fluid.executor import global_scope as global_scope
from ..fluid.executor import scope_guard as scope_guard
from ..fluid.framework import Program as Program
from ..fluid.framework import Variable as Variable
from ..fluid.framework import cpu_places as cpu_places
from ..fluid.framework import cuda_places as cuda_places
from ..fluid.framework import default_main_program as default_main_program
from ..fluid.framework import default_startup_program as default_startup_program
from ..fluid.framework import device_guard as device_guard
from ..fluid.framework import ipu_shard_guard as ipu_shard_guard
from ..fluid.framework import mlu_places as mlu_places
from ..fluid.framework import name_scope as name_scope
from ..fluid.framework import npu_places as npu_places
from ..fluid.framework import program_guard as program_guard
from ..fluid.framework import xpu_places as xpu_places
from ..fluid.io import load as load
from ..fluid.io import load_program_state as load_program_state
from ..fluid.io import save as save
from ..fluid.io import set_program_state as set_program_state
from ..fluid.layers import create_global_var as create_global_var
from ..fluid.layers import create_parameter as create_parameter
from ..fluid.layers.control_flow import Print as Print
from ..fluid.layers.metric_op import accuracy as accuracy
from ..fluid.layers.metric_op import auc as auc
from ..fluid.layers.nn import py_func as py_func
from ..fluid.optimizer import ExponentialMovingAverage as ExponentialMovingAverage
from ..fluid.parallel_executor import ParallelExecutor as ParallelExecutor
from ..fluid.param_attr import WeightNormParamAttr as WeightNormParamAttr
from .input import InputSpec as InputSpec
from .input import data as data
from .io import deserialize_persistables as deserialize_persistables
from .io import deserialize_program as deserialize_program
from .io import load_from_file as load_from_file
from .io import load_inference_model as load_inference_model
from .io import normalize_program as normalize_program
from .io import save_inference_model as save_inference_model
from .io import save_to_file as save_to_file
from .io import serialize_persistables as serialize_persistables
from .io import serialize_program as serialize_program



from __future__ import annotations

from typing import Any, Optional

from .common import OP_ROLE_KEY as OP_ROLE_KEY
from .common import OP_ROLE_VAR_KEY as OP_ROLE_VAR_KEY
from .common import CollectiveHelper as CollectiveHelper
from .common import OpRole as OpRole
from .common import is_backward_op as is_backward_op
from .common import is_optimizer_op as is_optimizer_op
from .common import is_update_op as is_update_op
from .meta_optimizer_base import MetaOptimizerBase as MetaOptimizerBase
from .sharding import utils as utils
from .sharding.fp16_helper import FP16Utils as FP16Utils
from .sharding.gradient_clip_helper import GradientClipHelper as GradientClipHelper
from .sharding.offload_helper import OffloadHelper as OffloadHelper
from .sharding.prune import ProgramDeps as ProgramDeps
from .sharding.shard import ProgramSegment as ProgramSegment
from .sharding.shard import Shard as Shard
from .sharding.utils import *
from .sharding.weight_decay_helper import WeightDecayHelper as WeightDecayHelper

logger: Any
formatter: Any
ch: Any

class ShardingOptimizer(MetaOptimizerBase):
    inner_opt: Any = ...
    meta_optimizers_white_list: Any = ...
    meta_optimizers_black_list: Any = ...
    mp_degree: int = ...
    def __init__(self, optimizer: Any) -> None: ...
    def minimize_impl(
        self,
        loss: Any,
        startup_program: Optional[Any] = ...,
        parameter_list: Optional[Any] = ...,
        no_grad_set: Optional[Any] = ...,
    ): ...
    def collect_segment(self, segment: Any, op_idx: Any, block: Any): ...
    def create_persistable_gradients_and_insert_merge_ops(
        self, main_block: Any, startup_block: Any, insert_idx: Any, grad_names: Any, shard: Any
    ) -> None: ...



from __future__ import annotations

from paddle.fluid import core as core
from paddle.fluid import dygraph_utils as dygraph_utils
from paddle.fluid.core import VarDesc as VarDesc
from paddle.fluid.data_feeder import check_dtype as check_dtype
from paddle.fluid.data_feeder import check_type as check_type
from paddle.fluid.data_feeder import (
    check_variable_and_dtype as check_variable_and_dtype,
)
from paddle.fluid.data_feeder import convert_dtype as convert_dtype
from paddle.fluid.framework import OpProtoHolder as OpProtoHolder
from paddle.fluid.framework import Variable as Variable
from paddle.fluid.framework import (
    convert_np_dtype_to_dtype_ as convert_np_dtype_to_dtype_,
)
from paddle.fluid.framework import default_main_program as default_main_program
from paddle.fluid.framework import device_guard as device_guard
from paddle.fluid.framework import dygraph_only as dygraph_only
from paddle.fluid.initializer import Constant as Constant
from paddle.fluid.layer_helper import LayerHelper as LayerHelper
from paddle.fluid.layers import fill_constant as fill_constant
from paddle.fluid.layers import scale as scale
from paddle.fluid.layers import utils as utils
from paddle.fluid.layers.layer_function_generator import templatedoc as templatedoc
from paddle.fluid.param_attr import ParamAttr as ParamAttr
from six.moves import reduce as reduce



from __future__ import annotations

from typing import Any, Optional

class DeviceType:
    CPU: str = ...
    GPU: str = ...
    XPU: str = ...
    NPU: str = ...
    MLU: str = ...

class Device:
    def __init__(self, dtype: Optional[Any] = ..., memory: str = ..., labels: str = ...) -> None: ...
    @property
    def dtype(self): ...
    @property
    def count(self): ...
    @property
    def memory(self): ...
    @property
    def labels(self): ...
    @labels.setter
    def labels(self, lbs: Any) -> None: ...
    def get_selected_device_key(self): ...
    def get_selected_devices(self, devices: str = ...): ...
    @classmethod
    def parse_device(cls): ...
    @classmethod
    def detect_device(cls): ...



from __future__ import annotations

from collections import OrderedDict as OrderedDict
from typing import Any, Optional

from .container import Container as Container
from .status import Status as Status

class PodSepc:
    def __init__(self) -> None: ...

class Pod(PodSepc):
    def __init__(self) -> None: ...
    def failed_container(self): ...
    @property
    def name(self): ...
    @property
    def replicas(self): ...
    @replicas.setter
    def replicas(self, r: Any) -> None: ...
    @property
    def rank(self): ...
    @rank.setter
    def rank(self, r: Any) -> None: ...
    @property
    def restart(self): ...
    @property
    def containers(self): ...
    def add_container(self, c: Any) -> None: ...
    @property
    def init_containers(self): ...
    def add_init_container(self, c: Any) -> None: ...
    @property
    def exit_code(self): ...
    def deploy(self) -> None: ...
    def stop(self, sigint: int = ...) -> None: ...
    def join(self) -> None: ...
    @property
    def status(self): ...
    def reset(self) -> None: ...
    def is_failed(self): ...
    def is_completed(self): ...
    def is_running(self): ...
    def logs(self, idx: Optional[Any] = ...) -> None: ...
    def tail(self, idx: Optional[Any] = ...) -> None: ...
    def watch(self, all_list: Any = ..., any_list: Any = ..., interval: int = ..., timeout: int = ...): ...



from __future__ import annotations

from typing import Any, Optional

class MetricRecord:
    def __init__(self, value: Any, step: Any) -> None: ...
    @property
    def value(self): ...
    @value.setter
    def value(self, value: Any) -> None: ...
    @property
    def step(self): ...
    @step.setter
    def step(self, step: Any) -> None: ...
    def mean(self): ...
    def get_state(self): ...
    @classmethod
    def from_state(cls, state: Any): ...
    def __eq__(self, other: Any) -> Any: ...

class MetricRecords:
    def __init__(self, direction: str = ...) -> None: ...
    @property
    def records(self): ...
    @records.setter
    def records(self, records: Any) -> None: ...
    @property
    def direction(self): ...
    @direction.setter
    def direction(self, direction: Any) -> None: ...
    def update(self, value: Any, step: int = ...) -> None: ...
    def get_best_value(self): ...
    def get_best_step(self): ...
    def get_statistics(self): ...
    def get_state(self): ...
    @classmethod
    def from_state(cls, state: Any): ...

class MetricsRecorder:
    def __init__(self, metrics: Optional[Any] = ...) -> None: ...
    @property
    def records(self): ...
    def exists(self, name: Any): ...
    def register_metrics(self, metrics: Optional[Any] = ...) -> None: ...
    def register(self, name: Any, direction: Optional[Any] = ...) -> None: ...
    def update(self, name: Any, value: Any, step: int = ...): ...
    def get_records(self, name: Any): ...
    def set_records(self, name: Any, records: Any) -> None: ...
    def get_best_value(self, name: Any): ...
    def get_best_step(self, name: Any): ...
    def get_statistics(self, name: Any): ...
    def get_state(self): ...
    @classmethod
    def from_state(cls, state: Any): ...



from __future__ import annotations

from typing import Any

class ConversionOptions:
    not_convert: Any = ...
    def __init__(self, not_convert: bool = ...) -> None: ...

def convert_call(func: Any): ...



from __future__ import annotations

from typing import Any, Optional

HOME: Any
DATA_HOME: Any

def must_mkdirs(path: Any) -> None: ...
def md5file(fname: Any): ...
def download(url: Any, module_name: Any, md5sum: Any, save_name: Optional[Any] = ...): ...
def fetch_all() -> None: ...
def split(reader: Any, line_count: Any, suffix: str = ..., dumper: Any = ...) -> None: ...
def cluster_files_reader(files_pattern: Any, trainer_count: Any, trainer_id: Any, loader: Any = ...): ...



from __future__ import annotations

from typing import Any, Optional

def amp_guard(
    enable: bool = ...,
    custom_white_list: Optional[Any] = ...,
    custom_black_list: Optional[Any] = ...,
    level: str = ...,
    dtype: str = ...,
) -> None: ...

class StateDictHook:
    def __init__(self, save_dtype: Any) -> None: ...
    def __call__(self, state_dict: Any) -> None: ...

def amp_decorate(
    models: Any,
    optimizers: Optional[Any] = ...,
    level: str = ...,
    master_weight: Optional[Any] = ...,
    save_dtype: Optional[Any] = ...,
): ...
